/************************************************************************
* \brief: NeoPhysIx - ultra fast 3D-physics simulation                  *
*																		*
* (c) copyright by Jrn Fischer											*
*                                                                       *
*        ***  ITS PROHIBITED TO DISTRIBUTE THIS SOURCE!  ***            *
*        ***************************************************            *
*																		* 
* @autor: Prof.Dr.Jrn Fischer											*
* @email: j.fischer@hs-mannheim.de										*
*                                                                       *
* @file : Evolution.cpp                                                 *
*************************************************************************/

#include "GlobalDefs.h"
#include "Math.h"
#include "NeuralNetwork.h"
#include "Evolution.h"

int LIFETIME=999999;

struct cell gene[NUM_OF_INDIVIDUUMS][NEURON_NUMBER];
FLOAT_32 fitness[NUM_OF_INDIVIDUUMS];


/*****************************************************************
* /brief this class simulates the evolutionary process
******************************************************************/

/*************************************************************
*  /brief copy is a helper function which copies gene i to t
**************************************************************/
void evolution::copy(int i, int t){
	fitness[t] = fitness[i];

	for (int k=0;k<NEURON_NUMBER; k++){
		for (int j=k; j<NEURON_NUMBER; j++){
			gene[t][k].weight[j] = gene[i][k].weight[j];
		}
		gene[t][k].bias = gene[i][k].bias;
	}
}

/*************************************************************
*  /brief swap is a helper function which swaps 2 genes
**************************************************************/
void evolution::swap(int i, int t){
	FLOAT_32 help;
 
	help = fitness[i];
	fitness[i] = fitness[t];
	fitness[t] = help;

	for (int k=0;k<NEURON_NUMBER; k++){
		for (int j=0; j<NEURON_NUMBER; j++){
		help = gene[i][k].weight[j];
		gene[i][k].weight[j] = gene[t][k].weight[j];
		gene[t][k].weight[j] =help;
		}
		help = gene[i][k].bias;
		gene[i][k].bias = gene[t][k].bias;
		gene[t][k].bias = help;
	}
}


/*************************************************************
* /brief constructor initializes the genes
**************************************************************/
evolution::evolution(){
	int t,i,k;

	for ( t=0; t<NUM_OF_INDIVIDUUMS; t++){
		fitness[t]=0;
		for (k=0;k<NEURON_NUMBER; k++){
			gene[t][k].output   = 0;
			gene[t][k].activity = 0;
			gene[t][k].bias     = 0;
			for (i=0;i<NEURON_NUMBER; i++){
				gene[t][k].weight[i] = 0;
			}
		}
	}

}

/*************************************************************
*  /brief setFitness is a function to evaluate the fitness of a 
*         gene
**************************************************************/
void evolution::setFitness(int numIndivid, FLOAT_32 value)
{
	fitness[numIndivid % NUM_OF_INDIVIDUUMS] = value;
}
/*************************************************************
*  /brief setFitness is a function to evaluate the fitness of a 
*         gene
**************************************************************/
FLOAT_32 evolution::getFitness(int numIndivid)
{
	return fitness[numIndivid % NUM_OF_INDIVIDUUMS];
}

/*************************************************************
*  /brief Sort() is a function which sorts the genes due to 
*         their fitness
**************************************************************/
void evolution::sort()
{
	int t,i;
	for ( t=0; t<NUM_OF_INDIVIDUUMS; t++){
		for (int k=0;k<NEURON_NUMBER; k++){
			gene[t][k].output   = 0;
			gene[t][k].activity = 0;
		}

		for ( i=t+1; i<NUM_OF_INDIVIDUUMS; i++){
			if (fitness[i]>=fitness[t]){
				swap(i,t);
			}
		}
	}

}
/*************************************************************
*  /brief Mutate mutates 1 synapse of the coded gene
**************************************************************/
void evolution::copyBest(){
	for (int t=0; t<NUM_OF_INDIVIDUUMS/2; t++){
		copy(t, NUM_OF_INDIVIDUUMS/2+t);
	}
}
/*************************************************************
*  /brief Mutate mutates 1 synapse of the coded gene
**************************************************************/
void evolution::Mutate()
{
		
	for (int t=1; t<NUM_OF_INDIVIDUUMS; t++){
		// mutate one synapse
		int k = pseudoRand()%NEURON_NUMBER;
		int j = pseudoRand()%NEURON_NUMBER;
		if (pseudoRand()%NEURON_NUMBER == 0){
		gene[t][k].bias      = (pseudoRand()%20000)/1000.0-10.0; // -10.0 .. 10.0 
		}
		else{
		double a=(pseudoRand()%20000)/1000.0-10.0; // -10.0 .. 10.0 
		gene[t][k].weight[j] = a;
		}
	}
	 
}
/*************************************************************
*  /brief Mutate mutates 1 synapse of the coded gene
**************************************************************/
void evolution::MutateSymmetric()
{
		
	for (int t=NUM_OF_INDIVIDUUMS/2; t<NUM_OF_INDIVIDUUMS; t++){
		// mutate one synapse
		int k = pseudoRand()%NEURON_NUMBER;
		int j = pseudoRand()%NEURON_NUMBER;
		if (pseudoRand()%NEURON_NUMBER == 0){
		gene[t][k].bias      = (pseudoRand()%20000)/1000.0-10.0; // -10.0 .. 10.0 
		}
		else{
		double a=(pseudoRand()%20000)/1000.0-10.0; // -10.0 .. 10.0 
		gene[t][k].weight[j] = a;
		if (pseudoRand()%100>50){
			gene[t][j].weight[k] = a;
		}
		else{
			gene[t][j].weight[k] = -a;
		}
		}
	}
	 
}
/*************************************************************
*  /brief creates a network from a gene
**************************************************************/
void evolution::createNetwork(int geneNumber)
{
		
	for (int k=0;k<NEURON_NUMBER; k++){
	    net.neuron[k].output   = 0;
		net.neuron[k].activity = 0;
		net.neuron[k].bias     = gene[geneNumber][k].bias;
		for (int j=0; j<NEURON_NUMBER; j++){
			net.neuron[k].weight[j] = gene[geneNumber][k].weight[j]; 
		}
		 
	}
	
}



