/************************************************************************
* \brief: NeoPhysIx - ultra fast 3D-physics simulation                  *
*																		*
* (c) copyright by Jrn Fischer											*
*                                                                       *
*        ***  ITS PROHIBITED TO DISTRIBUTE THIS SOURCE!  ***            *
*        ***************************************************            *
*																		* 
* @autor: Prof.Dr.Jrn Fischer											*
* @email: j.fischer@hs-mannheim.de										*
*                                                                       *
* @file : Evolution.h                                                   *
*************************************************************************/

#ifndef EVOLUTION_H
#define EVOLUTION_H

#define NUM_OF_INDIVIDUUMS 10

extern struct cell gene[NUM_OF_INDIVIDUUMS][NEURON_NUMBER];
extern FLOAT_32 fitness[NUM_OF_INDIVIDUUMS];

/*****************************************************************
* /brief this class simulates the evolutionary process
******************************************************************/
class evolution{
public:
	network net;
private:
	/*************************************************************
	*  /brief copy is a helper function which copies gene i to t
	**************************************************************/
	void copy(int i, int t);

	/*************************************************************
	*  /brief swap is a helper function which swaps 2 genes
	**************************************************************/
	void swap(int i, int t);


public:

	/*************************************************************
	* /brief constructor initializes the genes
	**************************************************************/
	evolution();

	/*************************************************************
	*  /brief setFitness is a function to evaluate the fitness of a 
	*         gene
	**************************************************************/
	void setFitness(int numIndivid, FLOAT_32 value);
	/*************************************************************
	*  /brief setFitness is a function to evaluate the fitness of a 
	*         gene
	**************************************************************/
	FLOAT_32 getFitness(int numIndivid);
	/*************************************************************
	*  /brief Sort() is a function which sorts the genes due to 
	*         their fitness
	**************************************************************/
	void sort();
	/*************************************************************
	*  /brief Mutate mutates 1 synapse of the coded gene
	**************************************************************/
	void copyBest();
	/*************************************************************
	*  /brief Mutate mutates 1 synapse of the coded gene
	**************************************************************/
	void Mutate();
	/*************************************************************
	*  /brief Mutate mutates 1 synapse of the coded gene
	**************************************************************/
	void MutateSymmetric();
	/*************************************************************
	*  /brief creates a network from a gene
	**************************************************************/
	void createNetwork(int geneNumber);


}; // end class

#endif // EVOLUTION_H
