/************************************************************************
* \brief: NeoPhysIx - ultra fast 3D-physics simulation                  *
*																		*
* (c) copyright by Jrn Fischer											*
*                                                                       *
*        ***  ITS PROHIBITED TO DISTRIBUTE THIS SOURCE!  ***            *
*        ***************************************************            *
*																		* 
* @autor: Prof.Dr.Jrn Fischer											*
* @email: j.fischer@hs-mannheim.de										*
*                                                                       *
* @file : Graphics.h                                                    *
*************************************************************************/
LONG FAR PASCAL AppWndProc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam);

void glPrint(int xx,int yy, int zz, char*text, float size);

/*************************************************************************
* /brief function to draw the landscape                                  *
**************************************************************************/
void drawEnvironment();
/*************************************************************************
* /brief function to draw the robot                                      *
**************************************************************************/
void drawObject(RigidBody* entity, float xPos, float yPos, float zPos);
/*************************************************************************
* /brief function to render the landscape and the robot                  *
**************************************************************************/
void Render(RigidBody* entity, int nr,char *text);
/*************************************************************************
* /brief function to SetupPixelFormat                                    *
**************************************************************************/
// Set up pixel format for graphics initialization

void SetupPixelFormat();

/******************************************************
* /brief : init graphics                              *
*******************************************************/

// Initialize OpenGL graphics
void InitGraphics();
/******************************************************
* /brief : function to resize  window                 *
*******************************************************/
// Resize graphics to fit window
void ResizeGraphics();
/************************************************************
* /brief function which handles a window resize or a destroy
**************************************************************/
// Handle window events and messages
LONG WINAPI MainWndProc (HWND hWnd, UINT uMsg, WPARAM  wParam, LPARAM  lParam);
/************************************************************
* /brief function which asks the keyboard and mouse actions
**************************************************************/
void evaluateMessages(MSG msg);
	
/************************************************************
* /brief main window function
**************************************************************/	
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow);
/************************************************************
* /brief initializes the viewpoint
**************************************************************/
void initViewPoint();
/************************************************************
* /brief steering of the graphical viewpoint
**************************************************************/
void mouseAction();
/************************************************************
* /brief build a quad
**************************************************************/
int buildQuad(const double &length);
/************************************************************
* /brief build a box
**************************************************************/
int buildBox(float width, float height, float depth);
/************************************************************
* /brief build a cylinder
**************************************************************/
int buildCylinder(float radius, float heigth);
/************************************************************
* /brief build a capsul
**************************************************************/
int buildCapsul(float radius, float heigth);
/************************************************************
* /brief build a sphere
**************************************************************/
int buildSphere(float xx, float yy, float zz, float radius);
/************************************************************
* /brief setColor to draw next
**************************************************************/
void setColor(float r, float g, float b);
