/************************************************************************
* \brief: NeoPhysIx - ultra fast 3D-physics simulation                  *
*																		*
* (c) copyright by Jrn Fischer											*
*                                                                       *
*        ***  ITS PROHIBITED TO DISTRIBUTE THIS SOURCE!  ***            *
*        ***************************************************            *
*																		* 
* @autor: Prof.Dr.Jrn Fischer											*
* @email: j.fischer@hs-mannheim.de										*
*                                                                       *
* @file : neuralNetworks.h                                              *
*************************************************************************/
#include "GlobalDefs.h"
#include "NeuralNetwork.h"

// -------------------------------------------------------------------------------
// --- init neurons --------------------------------------------------------------
// -------------------------------------------------------------------------------
void network::initNeurons()
{
	int t,r;

	for (t=0;t<NEURON_NUMBER;t++){
		for (r=0;r<NEURON_NUMBER;r++){
			neuron[t].weight[r]=0;
		}
		neuron[t].output   = 0.0f;
		neuron[t].activity = 0.0f;
		neuron[t].bias     = 0.0f;
		neuron[t].tau      = 0.4f;
	}
	neuron[0].weight[0]=1.5f;
	neuron[0].weight[1]=0.4f;
	neuron[0].bias=0.1f;

	neuron[1].weight[0]=-0.4f;
	neuron[1].weight[1]=1.5f;
	neuron[1].bias=0.1f;
}

// -------------------------------------------------------------------------------
// --- exp -----------------------------------------------------------------------
// -------------------------------------------------------------------------------
FLOAT_32 network::tanH(FLOAT_32 xx)
{
	FLOAT_32 yy;
	if (xx<0){
	yy=(1.0f+(-xx))+(-xx)*(-xx)+4.0f*(-xx)*(-xx)*(-xx);
	yy=2.0f/(1.0f+yy)-1.0f;
	}else{
	yy=(1.0f+(xx))+(xx)*(xx)+4.0f*(xx)*(xx)*(xx);
	yy=-(2.0f/(1.0f+yy)-1.0f);
	}
	return(yy);
}

// -------------------------------------------------------------------------------
// --- activate ------------------------------------------------------------------
// -------------------------------------------------------------------------------
void network::activate(FLOAT_32* input, int numOfInputs)
{
	int t,r;
	for (t=0; t<numOfInputs; t++){
		neuron[t].output = input[t];
	}
	// calculate activity
	for (t = numOfInputs; t < NEURON_NUMBER; t++){
		FLOAT_32 oldActivity = neuron[t].activity;
		neuron[t].activity = 0;
		for (r=0;r<NEURON_NUMBER;r++){
			neuron[t].activity+=neuron[t].weight[r]*neuron[r].output;
		}
		neuron[t].activity = oldActivity + neuron[t].tau * (neuron[t].activity + neuron[t].bias - oldActivity);
	}
	// calculate output
	for (t=numOfInputs; t<NEURON_NUMBER; t++){
		neuron[t].output = tanH(neuron[t].activity);//2.0/(1.0+exp(-2.0*neuron[t].activity))-1.0;
	}
}




